@@ -1,16 +1,35 @@
 ##############################################################################
-# Copyright (c) 2011-2014 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2011-2021, Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
-# Licensed under the New BSD License
-# (http://www.freebsd.org/copyright/freebsd-license.html)
 #
-# Python DB-API 2.0 module for Firebird. 
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# * Redistributions of source code must retain the above copyright notice, this
+#   list of conditions and the following disclaimer.
+#
+# * Redistributions in binary form must reproduce the above copyright notice,
+#  this list of conditions and the following disclaimer in the documentation
+#  and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#
+# Python DB-API 2.0 module for Firebird.
 ##############################################################################
 import sys
 
 PYTHON_MAJOR_VER = sys.version_info[0]
 
-DEFAULT_CHARSET='UTF8'
+DEFAULT_CHARSET = 'UTF8'
 
 ISC_TIME_SECONDS_PRECISION = 10000
 MAX_CHAR_LENGTH = 32767
@@ -101,6 +120,39 @@ isc_info_active_transactions = 109
 isc_info_active_tran_count = 110
 isc_info_creation_date = 111
 isc_info_db_file_size = 112
+fb_info_page_contents = 113
+fb_info_implementation = 114
+fb_info_page_warns = 115
+fb_info_record_warns = 116
+fb_info_bpage_warns = 117
+fb_info_dpage_warns = 118
+fb_info_ipage_warns = 119
+fb_info_ppage_warns = 120
+fb_info_tpage_warns = 121
+fb_info_pip_errors = 122
+fb_info_pip_warns = 123
+fb_info_pages_used = 124
+fb_info_pages_free = 125
+fb_info_ses_idle_timeout_db = 129
+fb_info_ses_idle_timeout_att = 130
+fb_info_ses_idle_timeout_run = 131
+fb_info_conn_flags = 132
+fb_info_crypt_key = 133
+fb_info_crypt_state = 134
+fb_info_statement_timeout_db = 135
+fb_info_statement_timeout_att = 136
+fb_info_protocol_version = 137
+fb_info_crypt_plugin = 138
+fb_info_creation_timestamp_tz = 139
+fb_info_wire_crypt = 140
+fb_info_features = 141
+fb_info_next_attachment = 142
+fb_info_next_statement = 143
+fb_info_db_guid = 144
+fb_info_db_file_id = 145
+fb_info_replica_mode = 146
+fb_info_username = 147
+fb_info_sqlrole = 148
 
 # isc_info_sql_records items
 isc_info_req_select_count = 13
@@ -142,6 +194,12 @@ SQL_TYPE_QUAD = 550
 SQL_TYPE_TIME = 560
 SQL_TYPE_DATE = 570
 SQL_TYPE_INT64 = 580
+SQL_TYPE_INT128 = 32752
+SQL_TYPE_TIMESTAMP_TZ = 32754
+SQL_TYPE_TIME_TZ = 32756
+SQL_TYPE_DEC_FIXED = 32758  # OBSOLATED
+SQL_TYPE_DEC64 = 32760
+SQL_TYPE_DEC128 = 32762
 SQL_TYPE_BOOLEAN = 32764
 SQL_TYPE_NULL = 32766
 
@@ -238,12 +296,19 @@ isc_dpb_auth_block = 79
 isc_dpb_client_version = 80
 isc_dpb_remote_protocol = 81
 isc_dpb_host_name = 82
-isc_dpb_os_user	 = 83
+isc_dpb_os_user = 83
 isc_dpb_specific_auth_data = 84
 isc_dpb_auth_plugin_list = 85
 isc_dpb_auth_plugin_name = 86
 isc_dpb_config = 87
 isc_dpb_nolinger = 88
+isc_dpb_reset_icu = 89
+isc_dpb_map_attach = 90
+isc_dpb_session_time_zone = 91
+isc_dpb_set_db_replica = 92
+isc_dpb_set_bind = 93
+isc_dpb_decfloat_round = 94
+isc_dpb_decfloat_traps = 95
 
 # Transaction Parameter Block parameter
 isc_tpb_version1 = 1
@@ -348,7 +413,7 @@ isc_spb_num_db = 6
 
 # isc_info_svc_db_stats params
 isc_spb_sts_data_pages = 0x01
-isc_spb_sts_db_log =0x02
+isc_spb_sts_db_log = 0x02
 isc_spb_sts_hdr_pages = 0x04
 isc_spb_sts_idx_pages = 0x08
 isc_spb_sts_sys_relations = 0x10
@@ -423,6 +488,14 @@ isc_info_sql_stmt_type = 21
 isc_info_sql_get_plan = 22
 isc_info_sql_records = 23
 isc_info_sql_batch_fetch = 24
+isc_info_sql_relation_alias = 25
+isc_info_sql_explain_plan = 26
+isc_info_sql_stmt_flags = 27
+isc_info_sql_stmt_timeout_user = 28
+isc_info_sql_stmt_timeout_run = 29
+isc_info_sql_stmt_blob_align = 30
+isc_info_sql_exec_path_blr_bytes = 31
+isc_info_sql_exec_path_blr_text = 32
 
 isc_info_sql_stmt_select = 1
 isc_info_sql_stmt_insert = 2
@@ -484,47 +557,47 @@ DSQL_drop = 2
 charset_map = {
     # DB CHAR SET NAME    :   PYTHON CODEC NAME (CANONICAL)
     # --------------------------------------------------------------------------
-    'OCTETS'              :   None, # Allow to pass through unchanged.
-    'UNICODE_FSS'         :   'utf_8',
-    'UTF8'                :   'utf_8', # (Firebird 2.0+)
-    'SJIS_0208'           :   'shift_jis',
-    'EUCJ_0208'           :   'euc_jp',
-    'DOS737'              :   'cp737',
-    'DOS437'              :   'cp437',
-    'DOS850'              :   'cp850',
-    'DOS865'              :   'cp865',
-    'DOS860'              :   'cp860',
-    'DOS863'              :   'cp863',
-    'DOS775'              :   'cp775',
-    'DOS862'              :   'cp862',
-    'DOS864'              :   'cp864',
-    'ISO8859_1'           :   'iso8859_1',
-    'ISO8859_2'           :   'iso8859_2',
-    'ISO8859_3'           :   'iso8859_3',
-    'ISO8859_4'           :   'iso8859_4',
-    'ISO8859_5'           :   'iso8859_5',
-    'ISO8859_6'           :   'iso8859_6',
-    'ISO8859_7'           :   'iso8859_7',
-    'ISO8859_8'           :   'iso8859_8',
-    'ISO8859_9'           :   'iso8859_9',
-    'ISO8859_13'          :   'iso8859_13',
-    'KSC_5601'            :   'euc_kr',
-    'DOS852'              :   'cp852',
-    'DOS857'              :   'cp857',
-    'DOS861'              :   'cp861',
-    'DOS866'              :   'cp866',
-    'DOS869'              :   'cp869',
-    'WIN1250'             :   'cp1250',
-    'WIN1251'             :   'cp1251',
-    'WIN1252'             :   'cp1252',
-    'WIN1253'             :   'cp1253',
-    'WIN1254'             :   'cp1254',
-    'BIG_5'               :   'big5',
-    'GB_2312'             :   'gb2312',
-    'WIN1255'             :   'cp1255',
-    'WIN1256'             :   'cp1256',
-    'WIN1257'             :   'cp1257',
-    'KOI8-R'              :   'koi8_r', # (Firebird 2.0+)
-    'KOI8-U'              :   'koi8_u', # (Firebird 2.0+)
-    'WIN1258'             :   'cp1258', # (Firebird 2.0+)
+    'OCTETS': None,   # Allow to pass through unchanged.
+    'UNICODE_FSS': 'utf_8',
+    'UTF8':   'utf_8',  # (Firebird 2.0+)
+    'SJIS_0208': 'shift_jis',
+    'EUCJ_0208': 'euc_jp',
+    'DOS737': 'cp737',
+    'DOS437': 'cp437',
+    'DOS850': 'cp850',
+    'DOS865': 'cp865',
+    'DOS860': 'cp860',
+    'DOS863': 'cp863',
+    'DOS775': 'cp775',
+    'DOS862': 'cp862',
+    'DOS864': 'cp864',
+    'ISO8859_1': 'iso8859_1',
+    'ISO8859_2': 'iso8859_2',
+    'ISO8859_3': 'iso8859_3',
+    'ISO8859_4': 'iso8859_4',
+    'ISO8859_5': 'iso8859_5',
+    'ISO8859_6': 'iso8859_6',
+    'ISO8859_7': 'iso8859_7',
+    'ISO8859_8': 'iso8859_8',
+    'ISO8859_9': 'iso8859_9',
+    'ISO8859_13': 'iso8859_13',
+    'KSC_5601': 'euc_kr',
+    'DOS852': 'cp852',
+    'DOS857': 'cp857',
+    'DOS861': 'cp861',
+    'DOS866': 'cp866',
+    'DOS869': 'cp869',
+    'WIN1250': 'cp1250',
+    'WIN1251': 'cp1251',
+    'WIN1252': 'cp1252',
+    'WIN1253': 'cp1253',
+    'WIN1254': 'cp1254',
+    'BIG_5': 'big5',
+    'GB_2312': 'gb2312',
+    'WIN1255': 'cp1255',
+    'WIN1256': 'cp1256',
+    'WIN1257': 'cp1257',
+    'KOI8R': 'koi8_r',   # (Firebird 2.0+)
+    'KOI8U': 'koi8_u',   # (Firebird 2.0+)
+    'WIN1258': 'cp1258',  # (Firebird 2.0+)
 }
