@@ -1,32 +1,51 @@
 ##############################################################################
-# Copyright (c) 2009-2014 Hajime Nakagami<nakagami@gmail.com>
+# Copyright (c) 2009-2021, Hajime Nakagami<nakagami@gmail.com>
 # All rights reserved.
-# Licensed under the New BSD License
-# (http://www.freebsd.org/copyright/freebsd-license.html)
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are met:
+#
+# * Redistributions of source code must retain the above copyright notice, this
+#   list of conditions and the following disclaimer.
+#
+# * Redistributions in binary form must reproduce the above copyright notice,
+#  this list of conditions and the following disclaimer in the documentation
+#  and/or other materials provided with the distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 # Python DB-API 2.0 module for Firebird.
 ##############################################################################
 from __future__ import print_function
 import sys
-import warnings
-import xdrlib, time, datetime, decimal, struct
+import time
+import datetime
+import decimal
 import itertools
 from collections import Mapping
-from firebirdsql.fberrmsgs import messages
-from firebirdsql import (DatabaseError, InternalError, OperationalError,
-    ProgrammingError, IntegrityError, DataError, NotSupportedError,)
-from firebirdsql.consts import *
-from firebirdsql.utils import *
-from firebirdsql.wireprotocol import WireProtocol, INFO_SQL_SELECT_DESCRIBE_VARS
+from firebirdsql import InternalError, OperationalError, NotSupportedError
+from firebirdsql.consts import *    # noqa
+from firebirdsql.utils import *     # noqa
+from firebirdsql.wireprotocol import WireProtocol
 from firebirdsql.socketstream import SocketStream
-from firebirdsql.xsqlvar import XSQLVAR, calc_blr, parse_select_items, parse_xsqlda
-__version__ = '0.9.6'
+from firebirdsql.xsqlvar import calc_blr, parse_xsqlda
+__version__ = '1.2.0'
 apilevel = '2.0'
 threadsafety = 1
 paramstyle = 'qmark'
 
 DEBUG = False
 
+
 def DEBUG_OUTPUT(*argv):
     if not DEBUG:
         return
@@ -34,6 +53,7 @@ def DEBUG_OUTPUT(*argv):
         print(s, end=' ', file=sys.stderr)
     print(file=sys.stderr)
 
+
 transaction_parameter_block = (
     # ISOLATION_LEVEL_READ_COMMITED_LEGACY
     bs([isc_tpb_version3, isc_tpb_write, isc_tpb_wait, isc_tpb_read_committed, isc_tpb_no_rec_version]),
@@ -44,34 +64,46 @@ transaction_parameter_block = (
     # ISOLATION_LEVEL_SERIALIZABLE
     bs([isc_tpb_version3, isc_tpb_write, isc_tpb_wait, isc_tpb_consistency]),
     # ISOLATION_LEVEL_READ_COMMITED_RO
-    bs([isc_tpb_version3, isc_tpb_read, isc_tpb_wait, isc_tpb_read_committed,isc_tpb_rec_version]),
+    bs([isc_tpb_version3, isc_tpb_read, isc_tpb_wait, isc_tpb_read_committed, isc_tpb_rec_version]),
 )
 
 Date = datetime.date
 Time = datetime.time
 TimeDelta = datetime.timedelta
 Timestamp = datetime.datetime
+
+
 def DateFromTicks(ticks):
-    return apply(Date,time.localtime(ticks)[:3])
+    return apply(Date, time.localtime(ticks)[:3])
+
+
 def TimeFromTicks(ticks):
-    return apply(Time,time.localtime(ticks)[3:6])
+    return apply(Time, time.localtime(ticks)[3:6])
+
+
 def TimestampFromTicks(ticks):
-    return apply(Timestamp,time.localtime(ticks)[:6])
+    return apply(Timestamp, time.localtime(ticks)[:6])
+
+
 def Binary(b):
     return bytes(b)
 
+
 class DBAPITypeObject:
-    def __init__(self,*values):
+    def __init__(self, *values):
         self.values = values
-    def __cmp__(self,other):
+
+    def __cmp__(self, other):
         if other in self.values:
             return 0
         if other < self.values:
             return 1
         else:
             return -1
+
+
 STRING = DBAPITypeObject(str)
-if PYTHON_MAJOR_VER==3:
+if PYTHON_MAJOR_VER == 3:
     BINARY = DBAPITypeObject(bytes)
 else:
     BINARY = DBAPITypeObject(str)
@@ -81,6 +113,7 @@ DATE = DBAPITypeObject(datetime.date)
 TIME = DBAPITypeObject(datetime.time)
 ROWID = DBAPITypeObject()
 
+
 class Statement(object):
     """
     statement handle and status (open/close)
@@ -112,8 +145,7 @@ class Statement(object):
                 self.handle, self.trans.trans_handle, sql)
             self.plan = None
 
-        if (self.trans.connection.accept_type == ptype_lazy_send
-                                and self.trans.connection.lazy_response_count):
+        if self.trans.connection.lazy_response_count:
             self.trans.connection.lazy_response_count -= 1
             (h, oid, buf) = self.trans.connection._op_response()
             self.handle = h
@@ -122,34 +154,31 @@ class Statement(object):
 
         i = 0
         if byte_to_int(buf[i]) == isc_info_sql_get_plan:
-            l = bytes_to_int(buf[i+1:i+3])
-            self.plan = self.trans.connection.bytes_to_str(buf[i+3:i+3+l])
-            i += 3 + l
-        self.stmt_type, self.xsqlda = parse_xsqlda(buf[i:],
-                                    self.trans.connection, self.handle)
-
-    def open(self):
-        DEBUG_OUTPUT("Statement::open()")
-        self._is_open = True
+            ln = bytes_to_int(buf[i+1:i+3])
+            self.plan = self.trans.connection.bytes_to_str(buf[i+3:i+3+ln])
+            i += 3 + ln
+        self.stmt_type, self.xsqlda = parse_xsqlda(buf[i:], self.trans.connection, self.handle)
+        if self.stmt_type == isc_info_sql_stmt_select:
+            self._is_open = True
 
     def close(self):
         DEBUG_OUTPUT("Statement::close()", self.handle)
-        if (self.stmt_type == isc_info_sql_stmt_select and self._is_open):
+        if self.stmt_type == isc_info_sql_stmt_select and self._is_open:
             self.trans.connection._op_free_statement(self.handle, DSQL_close)
-            if self.trans.connection.accept_type != ptype_lazy_send:
-                (h, oid, buf) = self.trans.connection._op_response()
-            else:
+            if self.trans.connection.accept_type == ptype_lazy_send:
                 self.trans.connection.lazy_response_count += 1
+            else:
+                (h, oid, buf) = self.trans.connection._op_response()
         self._is_open = False
 
     def drop(self):
-        DEBUG_OUTPUT("Statement::drop()")
-        if self.handle != -1:
+        DEBUG_OUTPUT("Statement::drop()", self.handle)
+        if self.handle != -1 and self._is_open:
             self.trans.connection._op_free_statement(self.handle, DSQL_drop)
-            if self.trans.connection.accept_type != ptype_lazy_send:
-                (h, oid, buf) = self.trans.connection._op_response()
-            else:
+            if self.trans.connection.accept_type == ptype_lazy_send:
                 self.trans.connection.lazy_response_count += 1
+            else:
+                (h, oid, buf) = self.trans.connection._op_response()
         self._is_open = False
         self.handle = -1
 
@@ -157,6 +186,7 @@ class Statement(object):
     def is_opened(self):
         return self._is_open and self.handle != -1
 
+
 class PreparedStatement(object):
     def __init__(self, cur, sql, explain_plan=False):
         DEBUG_OUTPUT("PreparedStatement::__init__()")
@@ -171,14 +201,18 @@ class PreparedStatement(object):
                 return None
             r = []
             for x in self.stmt.xsqlda:
-                r.append((x.aliasname, x.sqltype, x.display_length(),
-                        x.io_length(), x.precision(),
-                        x.sqlscale, True if x.null_ok else False))
+                r.append((
+                    x.aliasname, x.sqltype, x.display_length(),
+                    x.io_length(), x.precision(),
+                    x.sqlscale, True if x.null_ok else False))
             return r
         elif attrname == 'n_output_params':
             return len(self.stmt.xsqlda)
         raise AttributeError
 
+    def close(self):
+        self.stmt.close()
+
 
 def _fetch_generator(stmt):
     DEBUG_OUTPUT("_fetch_generator()", stmt.handle, stmt.trans._trans_handle)
@@ -186,10 +220,9 @@ def _fetch_generator(stmt):
     more_data = True
     while more_data:
         if not stmt.is_opened:
-            raise StopIteration()
+            return
         connection._op_fetch(stmt.handle, calc_blr(stmt.xsqlda))
-        (rows, more_data) = connection._op_fetch_response(
-                                        stmt.handle, stmt.xsqlda)
+        (rows, more_data) = connection._op_fetch_response(stmt.handle, stmt.xsqlda)
         for r in rows:
             # Convert BLOB handle to data
             for i in range(len(stmt.xsqlda)):
@@ -200,8 +233,8 @@ def _fetch_generator(stmt):
                     connection._op_open_blob(r[i], stmt.trans.trans_handle)
                     (h, oid, buf) = connection._op_response()
                     v = bs([])
-                    n = 1   # 1:mora data 2:no more data
-                    while n == 1:
+                    n = 1   # 0,1:mora data 2:no more data
+                    while n != 2:
                         connection._op_get_segment(h)
                         (n, oid, buf) = connection._op_response()
                         while buf:
@@ -209,10 +242,10 @@ def _fetch_generator(stmt):
                             v += buf[2:ln+2]
                             buf = buf[ln+2:]
                     connection._op_close_blob(h)
-                    if connection.accept_type != ptype_lazy_send:
-                        (h, oid, buf) = connection._op_response()
-                    else:
+                    if connection.accept_type == ptype_lazy_send:
                         connection.lazy_response_count += 1
+                    else:
+                        (h, oid, buf) = connection._op_response()
                     r[i] = v
                     if x.sqlsubtype == 1:    # TEXT
                         if connection.use_unicode:
@@ -220,8 +253,8 @@ def _fetch_generator(stmt):
                         else:
                             r[i] = connection.bytes_to_str(r[i])
             yield tuple(r)
-    DEBUG_OUTPUT("_fetch_generator() StopIteration()")
-    raise StopIteration()
+    return
+
 
 class Cursor(object):
     def __init__(self, trans):
@@ -243,8 +276,7 @@ class Cursor(object):
     def _convert_params(self, params):
         cooked_params = []
         for param in params:
-            if (type(param)==str or
-                (PYTHON_MAJOR_VER == 2 and type(param)==unicode)):
+            if type(param) == str or (PYTHON_MAJOR_VER == 2 and type(param) == unicode):
                 param = self.transaction.connection.str_to_bytes(param)
             cooked_params.append(param)
         return cooked_params
@@ -253,11 +285,11 @@ class Cursor(object):
         self.query = query
         if isinstance(query, PreparedStatement):
             stmt = query.stmt
-            stmt.close()
         else:
             if self.stmt:
-                self.stmt.close()
-            else:
+                self.stmt.drop()
+                self.stmt = None
+            if self.stmt is None:
                 self.stmt = Statement(self.transaction)
             stmt = self.stmt
             stmt.prepare(query)
@@ -265,45 +297,34 @@ class Cursor(object):
 
     def prep(self, query, explain_plan=False):
         DEBUG_OUTPUT("Cursor::prep()")
-        prepared_statement = PreparedStatement(self, query,
-                                                explain_plan=explain_plan)
+        prepared_statement = PreparedStatement(self, query, explain_plan=explain_plan)
         return prepared_statement
 
-    def execute(self, query, params=[]):
+    def execute(self, query, params=None):
+        if params is None:
+            params = []
         DEBUG_OUTPUT("Cursor::execute()", query, params)
         self.transaction.check_trans_handle()
         stmt = self._get_stmt(query)
         cooked_params = self._convert_params(params)
         if stmt.stmt_type == isc_info_sql_stmt_exec_procedure:
-            self.transaction.connection._op_execute2(stmt.handle,
+            self.transaction.connection._op_execute2(
+                stmt.handle,
                 self.transaction.trans_handle, cooked_params,
                 calc_blr(stmt.xsqlda))
-            self._callproc_result = \
-                self.transaction.connection._op_sql_response(stmt.xsqlda)
+            self._callproc_result = self.transaction.connection._op_sql_response(stmt.xsqlda)
             self.transaction.connection._op_response()
             self._fetch_records = None
         else:
-            DEBUG_OUTPUT("Cursor::execute() _op_execute()",
-                                stmt.handle, self.transaction.trans_handle)
-            self.transaction.connection._op_execute(stmt.handle,
-                                self.transaction.trans_handle, cooked_params)
-            try:
-                (h, oid, buf) = self.transaction.connection._op_response()
-            except OperationalError as e:
-                self._fetch_records = None
-                self._callproc_result = None
-                if e.gds_codes.intersection(
-                    [335544665, 335544466, 335544838, 335544347]):
-                    raise IntegrityError(e._message, e.gds_codes, e.sql_code)
-                if e.sql_code == -303:
-                    warnings.warn(e._message)
-                    return self
-                else:
-                    raise OperationalError(e._message, e.gds_codes, e.sql_code)
+            DEBUG_OUTPUT(
+                "Cursor::execute() _op_execute()",
+                stmt.handle, self.transaction.trans_handle)
+            self.transaction.connection._op_execute(
+                stmt.handle, self.transaction.trans_handle, cooked_params)
+            (h, oid, buf) = self.transaction.connection._op_response()
 
             if stmt.stmt_type == isc_info_sql_stmt_select:
                 self._fetch_records = _fetch_generator(stmt)
-                stmt.open()
             else:
                 self._fetch_records = None
             self._callproc_result = None
@@ -311,7 +332,9 @@ class Cursor(object):
 
         return self
 
-    def callproc(self, procname, params=[]):
+    def callproc(self, procname, params=None):
+        if params is None:
+            params = []
         DEBUG_OUTPUT("Cursor::callproc()")
         query = 'EXECUTE PROCEDURE ' + procname + ' ' + ','.join('?'*len(params))
         self.execute(query, params)
@@ -321,7 +344,6 @@ class Cursor(object):
         for params in seq_of_params:
             self.execute(query, params)
 
-
     def fetchone(self):
         if not self.transaction.is_dirty:
             return None
@@ -359,9 +381,9 @@ class Cursor(object):
             return None
         if not self._fetch_records:
             if self._callproc_result:
-                r = [tuple(self._callproc_result)]
+                proc_r = [tuple(self._callproc_result)]
                 self._callproc_result = None
-                return r
+                return proc_r
             return []
         # select statement
         return [tuple(r) for r in self._fetch_records]
@@ -420,9 +442,10 @@ class Cursor(object):
     def description(self):
         if not self.stmt:
             return None
-        return [(x.aliasname, x.sqltype, x.display_length(), x.io_length(),
-                    x.precision(), x.sqlscale, True if x.null_ok else False)
-                for x in self.stmt.xsqlda]
+        return [(
+            x.aliasname, x.sqltype, x.display_length(), x.io_length(),
+            x.precision(), x.sqlscale, True if x.null_ok else False
+        ) for x in self.stmt.xsqlda]
 
     @property
     def rowcount(self):
@@ -430,32 +453,44 @@ class Cursor(object):
         if self.stmt.handle == -1:
             return -1
 
-        self.transaction.connection._op_info_sql(self.stmt.handle,
-                                     bs([isc_info_sql_records]))
+        self.transaction.connection._op_info_sql(self.stmt.handle, bs([isc_info_sql_records]))
         (h, oid, buf) = self.transaction.connection._op_response()
-        assert buf[:3] == bs([0x17,0x1d,0x00]) # isc_info_sql_records
+        assert buf[:3] == bs([0x17, 0x1d, 0x00])    # isc_info_sql_records
         if self.stmt.stmt_type == isc_info_sql_stmt_select:
-            assert buf[17:20] == bs([0x0d,0x04,0x00])
-                                              # isc_info_req_select_count
+            assert buf[17:20] == bs([0x0d, 0x04, 0x00])     # isc_info_req_select_count
             count = bytes_to_int(buf[20:24])
         else:
             count = bytes_to_int(buf[27:31]) + bytes_to_int(buf[6:10]) + bytes_to_int(buf[13:17])
         DEBUG_OUTPUT("Cursor::rowcount()", self.stmt.stmt_type, count)
         return count
 
+
 class EventConduit(WireProtocol):
     def __init__(self, conn, names, timeout):
+        self.sock = None
         self.connection = conn
         self.event_names = {}
         for name in names:
             self.event_names[name] = 0
         self.timeout = timeout
-        (h, port, family, ip_address) = self.connection._op_connect_request()
+        self.connection._op_connect_request()
+        (h, oid, buf) = self.connection._op_response()
+        family = buf[:2]
+        port = bytes_to_bint(buf[2:4], u=True)
+        if family == b'\x02\x00':     # IPv4
+            ip_address = '.'.join([str(byte_to_int(c)) for c in buf[4:8]])
+        elif family == b'\x0a\x00':  # IPv6
+            address = bytes_to_hex(buf[8:24])
+            if not isinstance(address, str):    # Py3
+                address = address.decode('ascii')
+            ip_address = ':'.join(
+                [address[i: i+4] for i in range(0, len(address), 4)]
+            )
         self.sock = SocketStream(ip_address, port, timeout)
         self.connection.last_event_id += 1
         self.event_id = self.connection.last_event_id
 
-        self.connection._op_que_events(self.event_names, 0, 0, self.event_id)
+        self.connection._op_que_events(self.event_names, self.event_id)
         (h, oid, buf) = self.connection._op_response()
 
         (event_id, event_names) = self._wait_for_event(timeout=timeout)
@@ -463,7 +498,7 @@ class EventConduit(WireProtocol):
         self.event_names.update(event_names)
 
     def wait(self, timeout=None):
-        self.connection._op_que_events(self.event_names, 0, 0, self.event_id)
+        self.connection._op_que_events(self.event_names, self.event_id)
         (h, oid, buf) = self.connection._op_response()
 
         r = self._wait_for_event(timeout=timeout)
@@ -486,17 +521,18 @@ class EventConduit(WireProtocol):
         self.sock.close()
         self.sock = None
 
+
 class Connection(WireProtocol):
-    def cursor(self):
+    def cursor(self, factory=Cursor):
         DEBUG_OUTPUT("Connection::cursor()")
         if self._transaction is None:
             self.begin()
-        return Cursor(self._transaction)
+        return factory(self._transaction)
 
     def begin(self):
         DEBUG_OUTPUT("Connection::begin()")
         if not self.sock:
-            raise InternalError
+            raise InternalError("Missing socket")
         if self._transaction is None:
             self._transaction = Transaction(self, self._autocommit)
         self._transaction.begin()
@@ -522,42 +558,28 @@ class Connection(WireProtocol):
         self._op_exec_immediate(
             self._transaction.trans_handle, query=query)
         (h, oid, buf) = self._op_response()
-        self._transaction.is_dirty=True
+        self._transaction.is_dirty = True
 
-    def __init__(self, dsn=None, user=None, password=None, role=None, host=None,
-                    database=None, charset=DEFAULT_CHARSET, port=3050,
-                    page_size=4096, is_services=False, cloexec=False,
-                    timeout=None, isolation_level=None, use_unicode=None,
-                    auth_plugin_list=('Srp', 'Legacy_Auth'),
-                    wire_crypt=True, create_new=False):
+    def __init__(
+        self, dsn=None, user=None, password=None, role=None, host=None,
+        database=None, charset=DEFAULT_CHARSET, port=None,
+        page_size=4096, is_services=False, cloexec=False,
+        timeout=None, isolation_level=None, use_unicode=None,
+        auth_plugin_name=None, wire_crypt=True, create_new=False,
+        timezone=None
+    ):
         DEBUG_OUTPUT("Connection::__init__()")
+        if auth_plugin_name is None:
+            auth_plugin_name = 'Srp256'
+        WireProtocol.__init__(self)
+        self.sock = None
         self.db_handle = None
-        if dsn:
-            i = dsn.find(':')
-            if i < 0:
-                self.hostname = host
-                self.filename = dsn
-            else:
-                hostport = dsn[:i]
-                self.filename = dsn[i+1:]
-                i = hostport.find('/')
-                if i < 0:
-                    self.hostname = hostport
-                else:
-                    self.hostname = hostport[:i]
-                    port = int(hostport[i+1:])
-        else:
-            self.hostname = host
-            self.filename = database
-        if self.hostname is None:
-            self.hostname = 'localhost'
-        self.port = port
-        self.user = user
-        self.password = password
+        (self.hostname, self.port, self.filename, self.user, self.password) = \
+            parse_dsn(dsn, host, port, database, user, password)
         self.role = role
         self.charset = charset
         self.timeout = float(timeout) if timeout is not None else None
-        self.auth_plugin_list = auth_plugin_list
+        self.auth_plugin_name = auth_plugin_name
         self.wire_crypt = wire_crypt
         self.page_size = page_size
         self.is_services = is_services
@@ -566,25 +588,26 @@ class Connection(WireProtocol):
         else:
             self.isolation_level = int(isolation_level)
         self.use_unicode = use_unicode
+        self.timezone = timezone
         self.last_event_id = 0
 
         self._autocommit = False
         self._transaction = None
         self.sock = SocketStream(self.hostname, self.port, self.timeout, cloexec)
 
-        self._op_connect(auth_plugin_list, wire_crypt)
+        self._op_connect(auth_plugin_name, wire_crypt)
         try:
-            self._op_accept()
+            self._parse_connect_response()
         except OperationalError as e:
             self.sock.close()
             self.sock = None
             raise e
         if create_new:                      # create database
-            self._op_create(self.page_size)
+            self._op_create(self.timezone, self.page_size)
         elif self.is_services:                  # service api
             self._op_service_attach()
         else:                                   # connect
-            self._op_attach()
+            self._op_attach(self.timezone)
         (h, oid, buf) = self._op_response()
         self.db_handle = h
 
@@ -592,6 +615,11 @@ class Connection(WireProtocol):
         return self
 
     def __exit__(self, exc, value, traceback):
+        "On successful exit, commit. On exception, rollback. "
+        if exc:
+            self.rollback()
+        else:
+            self.commit()
         self.close()
 
     def set_isolation_level(self, isolation_level):
@@ -607,8 +635,7 @@ class Connection(WireProtocol):
         if info_requests[-1] == isc_info_end:
             self._op_info_database(bs(info_requests))
         else:
-            self._op_info_database(
-                        bs(info_requests+type(info_requests)([isc_info_end])))
+            self._op_info_database(bs(info_requests+type(info_requests)([isc_info_end])))
         (h, oid, buf) = self._op_response()
         i = 0
         i_request = 0
@@ -621,15 +648,15 @@ class Connection(WireProtocol):
             if req == isc_info_user_names:
                 user_names = []
                 while req == isc_info_user_names:
-                    l = bytes_to_int(buf[i+1:i+3])
-                    user_names.append(buf[i+3:i+3+l])
-                    i = i + 3 + l
+                    ln = bytes_to_int(buf[i+1:i+3])
+                    user_names.append(buf[i+3:i+3+ln])
+                    i = i + 3 + ln
                     req = byte_to_int(buf[i])
                 r.append((req, user_names))
             else:
-                l = bytes_to_int(buf[i+1:i+3])
-                r.append((req, buf[i+3:i+3+l]))
-                i = i + 3 + l
+                ln = bytes_to_int(buf[i+1:i+3])
+                r.append((req, buf[i+3:i+3+ln]))
+                i = i + 3 + ln
             i_request += 1
         return r
 
@@ -700,7 +727,7 @@ class Connection(WireProtocol):
             mm = (5 * dd - 3) // 153
             dd = 5 * dd - 3 - 153 * mm
             dd = (dd + 5) // 5
-            yyyy = 100 * century + nday;
+            yyyy = 100 * century + nday
             if mm < 10:
                 mm += 3
             else:
@@ -731,8 +758,7 @@ class Connection(WireProtocol):
                 if rs[i][0] == isc_info_error:
                     results[info_requests[i]] = None
                 else:
-                    results[info_requests[i]] = self._db_info_convert_type(
-                                                    info_requests[i], rs[i][1])
+                    results[info_requests[i]] = self._db_info_convert_type(info_requests[i], rs[i][1])
             return results
 
     def trans_info(self, info_requests):
@@ -772,6 +798,7 @@ class Connection(WireProtocol):
     def is_disconnect(self):
         return self.sock is None
 
+
 class Transaction(object):
     def __init__(self, connection, is_autocommit=False):
         DEBUG_OUTPUT("Transaction::__init__()")
@@ -785,10 +812,9 @@ class Transaction(object):
             tpb += bs([isc_tpb_autocommit])
         self.connection._op_transaction(tpb)
         (h, oid, buf) = self.connection._op_response()
-        self._trans_handle = h
-        DEBUG_OUTPUT("Transaction::_begin()",
-            self._trans_handle,
-            self.connection)
+        self._trans_handle = None if h < 0 else h
+        DEBUG_OUTPUT(
+            "Transaction::_begin()", self._trans_handle, self.connection)
         self.is_dirty = False
 
     def begin(self):
@@ -798,16 +824,12 @@ class Transaction(object):
     def savepoint(self, name):
         if self._trans_handle is None:
             return
-        self.connection._op_exec_immediate(self._trans_handle,
-                        query='SAVEPOINT '+name)
+        self.connection._op_exec_immediate(self._trans_handle, query='SAVEPOINT '+name)
         (h, oid, buf) = self.connection._op_response()
 
     def commit(self, retaining=False):
-        DEBUG_OUTPUT("Transaction::commit()",
-                        self._trans_handle,
-                        self,
-                        self.connection,
-                        retaining)
+        DEBUG_OUTPUT(
+            "Transaction::commit()", self._trans_handle, self, self.connection, retaining)
         if self._trans_handle is None:
             return
         if not self.is_dirty:
@@ -822,17 +844,14 @@ class Transaction(object):
         self.is_dirty = False
 
     def rollback(self, retaining=False, savepoint=None):
-        DEBUG_OUTPUT("Transaction::rollback()",
-                        self._trans_handle,
-                        self,
-                        self.connection,
-                        retaining,
-                        savepoint)
+        DEBUG_OUTPUT(
+            "Transaction::rollback()", self._trans_handle, self,
+            self.connection, retaining, savepoint)
         if self._trans_handle is None:
             return
         if savepoint:
-            self.connection._op_exec_immediate(self._trans_handle,
-                        query='ROLLBACK TO '+savepoint)
+            self.connection._op_exec_immediate(
+                self._trans_handle, query='ROLLBACK TO '+savepoint)
             (h, oid, buf) = self.connection._op_response()
             return
         if not self.is_dirty:
@@ -848,11 +867,10 @@ class Transaction(object):
 
     def _trans_info(self, info_requests):
         if info_requests[-1] == isc_info_end:
-            self.connection._op_info_transaction(self.trans_handle,
-                    bs(info_requests))
+            self.connection._op_info_transaction(self.trans_handle, bs(info_requests))
         else:
-            self.connection._op_info_transaction(self.trans_handle,
-                    bs(info_requests+type(info_requests)([isc_info_end])))
+            self.connection._op_info_transaction(
+                self.trans_handle, bs(info_requests+type(info_requests)([isc_info_end])))
         (h, oid, buf) = self.connection._op_response()
         i = 0
         i_request = 0
@@ -862,9 +880,9 @@ class Transaction(object):
             if req == isc_info_end:
                 break
             assert req == info_requests[i_request] or req == isc_info_error
-            l = bytes_to_int(buf[i+1:i+3])
-            r.append((req, buf[i+3:i+3+l]))
-            i = i + 3 + l
+            ln = bytes_to_int(buf[i+1:i+3])
+            r.append((req, buf[i+3:i+3+ln]))
+            i = i + 3 + ln
 
             i_request += 1
         return r
@@ -948,4 +966,3 @@ class RowMapping(Mapping):
                   for desc in self._description]
         return ("<RowMapping at 0x%08x with fields: %s>" %
                 (id(self), ", ".join(values)))
-
